@@ -84,6 +84,9 @@ static gboolean interrupted = FALSE;
 static gchar *ParsedAutomaticLogin = NULL;
 static gchar *ParsedTimedLogin = NULL;
 
+int greeter_fd_out = -1;
+int greeter_fd_in = -1;
+
 extern gboolean gdm_first_login;
 extern gboolean gdm_emergency_server;
 extern pid_t extra_process;
@@ -190,6 +193,18 @@ ignore_xerror_handler (Display *disp, XErrorEvent *evt)
 	return 0;
 }
 
+static void
+whack_greeter_fds (void)
+{
+	if (greeter_fd_out > 0)
+		close (greeter_fd_out);
+	greeter_fd_out = -1;
+	if (greeter_fd_in > 0)
+		close (greeter_fd_in);
+	greeter_fd_in = -1;
+}
+
+
 void 
 gdm_slave_start (GdmDisplay *display)
 {  
@@ -398,6 +413,8 @@ gdm_slave_whack_greeter (void)
 		ve_waitpid_no_signal (d->greetpid, 0, 0); 
 	d->greetpid = 0;
 
+	whack_greeter_fds ();
+
 	gdm_slave_send_num (GDM_SOP_GREETPID, 0);
 
 	gdm_sigchld_block_pop ();
@@ -877,6 +894,8 @@ restart_the_greeter (void)
 			ve_waitpid_no_signal (d->greetpid, 0, 0); 
 		d->greetpid = 0;
 
+		whack_greeter_fds ();
+
 		gdm_slave_send_num (GDM_SOP_GREETPID, 0);
 
 		gdm_sigchld_block_pop ();
@@ -1272,12 +1291,12 @@ run_pictures (void)
 		}
 		g_free (ret);
 
-		gdm_fdprintf (STDOUT_FILENO, "%c", STX);
+		gdm_fdprintf (greeter_fd_out, "%c", STX);
 
 		i = 0;
 		while ((bytes = fread (buf, sizeof (char),
 				       sizeof (buf), fp)) > 0) {
-			write (STDOUT_FILENO, buf, bytes);
+			write (greeter_fd_out, buf, bytes);
 			i += bytes;
 		}
 
@@ -1337,12 +1356,9 @@ gdm_slave_greeter (void)
 	/* Plumbing */
 	close (pipe1[1]);
 	close (pipe2[0]);
-	
-	if (pipe1[0] != STDIN_FILENO) 
-	    dup2 (pipe1[0], STDIN_FILENO);
-	
-	if (pipe2[1] != STDOUT_FILENO) 
-	    dup2 (pipe2[1], STDOUT_FILENO);
+
+	dup2 (pipe1[0], STDIN_FILENO);
+	dup2 (pipe2[1], STDOUT_FILENO);
 
 	closelog ();
 
@@ -1494,18 +1510,10 @@ gdm_slave_greeter (void)
 	fcntl(pipe1[1], F_SETFD, fcntl(pipe1[1], F_GETFD, 0) | FD_CLOEXEC);
 	fcntl(pipe2[0], F_SETFD, fcntl(pipe2[0], F_GETFD, 0) | FD_CLOEXEC);
 
-	/* flush our input before we change the piping */
-	fflush (stdin);
+	whack_greeter_fds ();
 
-	if (pipe1[1] != STDOUT_FILENO) {
-	    dup2 (pipe1[1], STDOUT_FILENO);
-	    close (pipe1[1]);
-	}
-	
-	if (pipe2[0] != STDIN_FILENO)  {
-	    dup2 (pipe2[0], STDIN_FILENO);
-	    close (pipe2[0]);
-	}
+	greeter_fd_out = pipe1[1];
+	greeter_fd_in = pipe2[0];
 	
 	gdm_debug ("gdm_slave_greeter: Greeter on pid %d", (int)pid);
 
@@ -2213,12 +2221,6 @@ session_child_run (struct passwd *pwent,
 	if ( ! def_language) {
 		gnome_setenv ("LANG", language, TRUE);
 		gnome_setenv ("GDM_LANG", language, TRUE);
-	} else {
-		if (g_getenv ("LANG") == NULL)
-			gnome_unsetenv ("GDM_LANG");
-		else
-			/* setusercontext sets up user languages */
-			gnome_setenv ("GDM_LANG", g_getenv ("LANG"), TRUE);
 	}
 #else
 	if (setuid (pwent->pw_uid) < 0) 
@@ -2227,7 +2229,11 @@ session_child_run (struct passwd *pwent,
 
 	/* Set locale */
 	gnome_setenv ("LANG", language, TRUE);
-	gnome_setenv ("GDM_LANG", language, TRUE);
+	/* Only force GDM_LANG to something if there is other then
+	 * default selected.  Else let the session do whatever it
+	 * does since we're using sys default */
+	if ( ! def_language)
+		gnome_setenv ("GDM_LANG", language, TRUE);
 #endif
 	
 	chdir (home_dir);
@@ -2986,6 +2992,7 @@ gdm_slave_child_handler (int sig)
 		    WEXITSTATUS (status) == DISPLAY_RESTARTGREETER) {
 			greet = FALSE;
 			d->greetpid = 0;
+			whack_greeter_fds ();
 			gdm_slave_send_num (GDM_SOP_GREETPID, 0);
 
 			if (restart_greeter_now) {
@@ -2999,6 +3006,8 @@ gdm_slave_child_handler (int sig)
 			return;
 		}
 
+		whack_greeter_fds ();
+
 		/* just for paranoia's sake */
 		seteuid (0);
 		setegid (0);
@@ -3078,6 +3087,30 @@ gdm_slave_usr2_handler (int sig)
 					g_list_append (unhandled_notifies,
 						       g_strdup (&s[1]));
 			}
+		} else if (s[0] == GDM_SLAVE_NOTIFY_COMMAND) {
+			if (strcmp (&s[1], GDM_NOTIFY_DIRTY_SERVERS) == 0) {
+				/* never restart flexi servers
+				 * they whack themselves */
+				if (d->type != TYPE_FLEXI_XNEST &&
+				    d->type != TYPE_FLEXI)
+					remanage_asap = TRUE;
+			} else if (strcmp (&s[1], GDM_NOTIFY_SOFT_RESTART_SERVERS) == 0) {
+				/* never restart flexi servers,
+				 * they whack themselves */
+				/* FIXME: here we should handle actual
+				 * restarts of flexi servers, but it probably
+				 * doesn't matter */
+				if (d->type != TYPE_FLEXI_XNEST &&
+				    d->type != TYPE_FLEXI) {
+					if ( ! d->logged_in) {
+						gdm_server_stop (d);
+						gdm_verify_cleanup (d);
+						_exit (DISPLAY_REMANAGE);
+					} else {
+						remanage_asap = TRUE;
+					}
+				}
+			}
 		}
 	}
 
@@ -3206,18 +3239,18 @@ gdm_slave_greeter_ctl (char cmd, const char *str)
     check_notifies_immediately ++;
 
     if ( ! ve_string_empty (str)) {
-	    gdm_fdprintf (STDOUT_FILENO, "%c%c%s\n", STX, cmd, str);
+	    gdm_fdprintf (greeter_fd_out, "%c%c%s\n", STX, cmd, str);
     } else {
-	    gdm_fdprintf (STDOUT_FILENO, "%c%c\n", STX, cmd);
+	    gdm_fdprintf (greeter_fd_out, "%c%c\n", STX, cmd);
     }
 
     /* Skip random junk that might have accumulated */
     do {
-	    c = gdm_fdgetc (STDIN_FILENO);
+	    c = gdm_fdgetc (greeter_fd_in);
     } while (c != EOF && c != STX);
     
     if (c == EOF ||
-	(buf = gdm_fdgets (STDIN_FILENO)) == NULL) {
+	(buf = gdm_fdgets (greeter_fd_in)) == NULL) {
 	    check_notifies_immediately --;
 	    interrupted = TRUE;
 	    /* things don't seem well with the greeter, it probably died */
