@@ -210,6 +210,9 @@ greeter_action_language (GreeterItemInfo *info,
       gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox),
 			  label, FALSE, FALSE, 0);
       view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (lang_model));
+      /* FIXME: we should handle this better, but things really look
+       * like crap if we aren't always LTR */
+      gtk_widget_set_direction (view, GTK_TEXT_DIR_LTR);
       gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (view), FALSE);
       gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (view),
 					       GTK_DIALOG_MODAL,
