@@ -3,6 +3,7 @@
 #include "greeter_item_pam.h"
 #include "greeter_parser.h"
 #include "gdm.h"
+#include "gdmwm.h"
 #include "vicious.h"
 #include <string.h>
 
@@ -23,10 +24,8 @@ greeter_item_pam_set_user (const char *user)
 static void
 user_pw_activate (GtkEntry *entry, GreeterItemInfo *info)
 {
-  static gboolean first_return = TRUE;
   char *tmp;
   GreeterItemInfo *error_info;
-  GreeterItemInfo *message_info;
   
   gtk_widget_set_sensitive (GTK_WIDGET (entry), FALSE);
 
@@ -40,16 +39,6 @@ user_pw_activate (GtkEntry *entry, GreeterItemInfo *info)
     greeter_current_user = g_strdup (gtk_entry_get_text (GTK_ENTRY (entry)));
   }
   
-  /* somewhat ugly thing to clear the initial message */
-  if (first_return) {
-    first_return = FALSE;
-    message_info = greeter_lookup_id ("pam-message");
-    if (message_info)
-      g_object_set (G_OBJECT (message_info->item),
-		    "text", "",
-		    NULL);
-  }
-  
   /* clear the err_box */
   if (err_box_clear_handler > 0)
     {
@@ -70,6 +59,28 @@ user_pw_activate (GtkEntry *entry, GreeterItemInfo *info)
   g_free (tmp);
 }
 
+static gboolean
+key_press_handler (GtkWidget *widget, GdkEventKey *event)
+{
+	if (event == NULL)
+		return FALSE;
+
+	switch (event->keyval) {
+
+	case GDK_Up:
+	case GDK_Down:
+	case GDK_Tab:
+		g_signal_stop_emission_by_name (G_OBJECT (widget),
+						"key_press_event");
+		return TRUE;
+
+	default:
+		break;
+	}
+
+	return FALSE;
+}
+
 gboolean
 greeter_item_pam_setup (void)
 {
@@ -94,6 +105,9 @@ greeter_item_pam_setup (void)
 
       g_signal_connect (entry, "activate",
 			GTK_SIGNAL_FUNC (user_pw_activate), entry_info);
+      g_signal_connect (entry, "key_press_event", 
+			G_CALLBACK (key_press_handler),
+			NULL);
     }
   return TRUE;
 }
