@@ -1,7 +1,7 @@
 /* Misc routines
  *
  * (c) 2000 Eazel, Inc.
- * (c) 2001 George Lebl
+ * (c) 2001,2002 George Lebl
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -21,6 +21,11 @@
 #ifndef VE_MISC_H
 #define VE_MISC_H
 
+#include <glib.h>
+#include <time.h>
+#include <sys/types.h>
+#include <sys/wait.h>
+
 char **	ve_split (const char *s);
 char *	ve_first_word (const char *s);
 char *  ve_rest (const char *s);
@@ -37,6 +42,12 @@ gboolean ve_is_string_in_list_case_no_locale (const GList *list,
 #define ve_string_empty(x) ((x)==NULL||(x)[0]=='\0')
 #define ve_sure_string(x) ((x)!=NULL?(x):"")
 
+/* Find a file using the specified list of directories, an absolute path is
+ * just checked, whereas a relative path is search in the given directories,
+ */
+char *		ve_find_file_simple		(const char *filename,
+						 const GList *directories);
+
 /* Find a file using the specified list of directories, an absolute path is
  * just checked, whereas a relative path is search in the given directories,
  * gnome_datadir, g_get_prgname() subdirectory, in GNOME_PATH
@@ -44,4 +55,17 @@ gboolean ve_is_string_in_list_case_no_locale (const GList *list,
 char *		ve_find_file			(const char *filename,
 						 const GList *directories);
 
+/* These two functions will ALWAYS return a non-NULL string,
+ * if there is an error, they return the unconverted string */
+char * ve_locale_to_utf8 (const char *str);
+char * ve_locale_from_utf8 (const char *str);
+
+/* works with utf-8 strings and always returns something */
+char * ve_strftime (struct tm *the_tm, const char *format);
+
+/* function which doesn't stop on signals */
+pid_t ve_waitpid_no_signal (pid_t pid, int *status, int options);
+
+
+
 #endif /* VE_MISC_H */
